/*  TA3D, a remake of Total Annihilation
    Copyright (C) 2005   Roland BROCHARD

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA*/

/*----------------------------------------------------------\
|                         UnitEngine.h                      |
|    Contains the unit engine, which simulates units and    |
| make them interact with each other.                       |
|                                                           |
\----------------------------------------------------------*/

#ifndef __UNITENGINE_CLASS
#define __UNITENGINE_CLASS

#define MAX_UNIT_PER_PLAYER		2000		// 250 Unités par joueur maximum

#include "fbi.h"
#include "weapons.h"

void *create_unit(int type_id,int owner,VECTOR pos,MAP *map);

struct MISSION			// Structure pour stocker les ordres
{
	uint8		mission;
	PATH_NODE	*path;		// Chemin emprunté par l'unité si besoin pour la mission
	VECTOR		target;
	MISSION 	*next;		// Mission suivante
	bool		step;		// Etape d'une mission
	float		time;		// Temps écoulé depuis la création de l'ordre
	int			data;		// Données de l'ordre
	byte		flags;		// Données supplémentaires
	float		last_d;		// Dernière distance enregistrée
	void		*p;
};

class SCRIPT_ENV_STACK			// Pile pour la gestion des scripts
{
public:
	int					var[15];
	uint32				signal_mask;
	sint32				cur;
	SCRIPT_ENV_STACK	*next;

	inline void init()
	{
		for(uint8 i=0;i<15;i++)
			var[i]=0;
		cur=0;
		signal_mask=0;
		next=NULL;
	}

	inline SCRIPT_ENV_STACK()
	{
		init();
	}
};

class SCRIPT_ENV			// Classe pour la gestion de l'environnement des scripts
{
public:
	SCRIPT_STACK		*stack;			// Pile utilisée par les scripts
	SCRIPT_ENV_STACK	*env;			// Pile d'éxecution des scripts
	float				wait;
	bool				running;

	inline void init()
	{
		stack=NULL;
		env=NULL;
		wait=0.0f;
		running=false;
	}

	SCRIPT_ENV()
	{
		init();
	}

	inline void destroy()
	{
		while(stack) {
			SCRIPT_STACK *next=stack->next;
			delete stack;
			stack=next;
			}
		while(env) {
			SCRIPT_ENV_STACK *next=env->next;
			delete env;
			env=next;
			}
		init();
	}

	inline ~SCRIPT_ENV()
	{
		destroy();
	}

	inline void push(int v)
	{
		SCRIPT_STACK *new_stack=new SCRIPT_STACK;
		new_stack->next=stack;
		stack=new_stack;
		stack->val=v;
	}

	inline int pop()
	{
		if(stack==NULL) {			// Si la pile est vide, renvoie 0 et un message pour le débuggage
#ifdef DEBUG_MODE
			Console->AddEntry("interpréteur de scripts: pile vide\n");
#endif
			return 0;
			}
		int v=stack->val;
		SCRIPT_STACK *old=stack;
		stack=stack->next;
		delete old;
		return v;
	}
};

#define SCRIPT_QueryPrimary		0x00
#define SCRIPT_AimPrimary		0x01
#define SCRIPT_FirePrimary		0x02
#define SCRIPT_QuerySecondary	0x03
#define SCRIPT_AimSecondary		0x04
#define SCRIPT_FireSecondary	0x05
#define SCRIPT_QueryTertiary	0x06
#define SCRIPT_AimTertiary		0x07
#define SCRIPT_FireTertiary		0x08
#define SCRIPT_TargetCleared	0x09
#define SCRIPT_stopbuilding		0x0A
#define SCRIPT_stop				0x0B
#define SCRIPT_startbuilding	0x0C
#define SCRIPT_go				0x0D
#define SCRIPT_killed			0x0E
#define SCRIPT_StopMoving		0x0F
#define SCRIPT_Deactivate		0x10
#define SCRIPT_Activate			0x11
#define SCRIPT_create			0x12
#define SCRIPT_MotionControl	0x13
#define SCRIPT_startmoving		0x14
#define SCRIPT_MoveRate1		0x15
#define SCRIPT_MoveRate2		0x16
#define SCRIPT_MoveRate3		0x17
#define SCRIPT_RequestState		0x18
#define SCRIPT_TransportPickup	0x19
#define SCRIPT_TransportDrop	0x1A
#define SCRIPT_QueryTransport	0x1B
#define SCRIPT_BeginTransport	0x1C
#define SCRIPT_EndTransport		0x1D
#define SCRIPT_SetSpeed			0x1E
#define SCRIPT_SetDirection		0x1F
#define SCRIPT_SetMaxReloadTime	0x20
#define SCRIPT_QueryBuildInfo	0x21
#define NB_SCRIPT				0x22

class UNIT				// Classe pour la gestion des unités
{
public:
	SCRIPT			*script;		// Scripts concernant l'unité
	int				*s_var;			// Tableau de variables pour les scripts
	MODEL			*model;			// Modèle représentant l'objet
	byte			owner_id;		// Numéro du propriétaire de l'unité
	short			type_id;		// Type d'unité
	float			hp;				// Points de vide restant à l'unité
	VECTOR			Pos;			// Vecteur position
	VECTOR			V;				// Vitesse de l'unité
	VECTOR			Angle;			// Orientation dans l'espace
	VECTOR			V_Angle;		// Variation de l'orientation dans l'espace
	bool			sel;			// Unité sélectionnée?
	SCRIPT_DATA		data;			// Données pour l'animation de l'unité par le script
	float			*port;			// Ports
	MISSION			*mission;		// Ordres de l'unité
	byte			nb_running;		// Nombre de scripts lancés en même temps
	SCRIPT_ENV		*script_env;	// Environnements des scripts
	byte			flags;			// Pour indiquer entre autres au gestionnaire d'unités si l'unité existe
	float			c_time;			// Compteur de temps entre 2 émissions de particules par une unité de construction
	bool			compute_coord;	// Indique s'il est nécessaire de recalculer les coordonnées du modèle 3d
	short			*script_val;	// Tableau de valeurs retournées par les scripts
	uint16			idx;			// Indice dans le tableau d'unité
	VECTOR			aim_dir;		// Vecteur de visée
	float			h;				// Altitude (par rapport au sol)
	bool			visible;		// Indique si l'unité est visible / Tell if the unit is currently visible
	short			burst[3];		// Pour la gestion des tirs
	float			b_delay[3];
	short			stock[3];		// Stock de munitions si besoin d'en fabriquer / ammo stock
	short			groupe;			// Indique si l'unité fait partie d'un groupe
	bool			built;			// Indique si l'unité est en cours de construction (par une autre unité)
	bool			attacked;		// Indique si l'unité est attaquée
	float			planned_weapons;	// Armes en construction / all is in the name
	int				*memory;	// Pour se rappeler sur quelles armes on a déjà tiré
	byte			mem_size;
	char			*script_idx;	// Index of scripts to prevent multiple search
	bool			attached;
	short			*attached_list;
	short			*link_list;
	byte			nb_attached;
	bool			just_created;
	int				severity;
	sint32			old_px;			// Old position on the map
	sint32			old_py;			// Used to recover fog of war
	uint16			old_sight;
	sint32			cur_px;
	sint32			cur_py;
	float			metal_prod;
	float			metal_cons;
	float			energy_prod;
	float			energy_cons;
	uint32			last_time_sound;	// Remember last time it played a sound, so we don't get a unit SHOUTING for a simple move

	inline bool do_nothing()
	{
		return mission==NULL || ((mission->mission==MISSION_STOP || mission->mission==MISSION_STANDBY || mission->mission==MISSION_VTOL_STANDBY) && mission->next==NULL);
	}

	inline void start_mission_script(int mission_type)
	{
		if(script==NULL)	return;
		switch(mission_type)
		{
		case MISSION_ATTACK:
		case MISSION_AIM:
		case MISSION_SHOOT:
//			activate();
			break;
		case MISSION_PATROL:
		case MISSION_MOVE_CALC_PATH:
		case MISSION_MOVE:
			launch_script(get_script_index(SCRIPT_MotionControl));
			launch_script(get_script_index(SCRIPT_startmoving));
			if(nb_attached==0)
				launch_script(get_script_index(SCRIPT_MoveRate1));		// Pour l'atlas
/*			launch_script(get_script_index("MotionControl"));
			launch_script(get_script_index("startmoving"));
			launch_script(get_script_index("MoveRate1"));		// Pour l'atlas*/
			break;
		case MISSION_BUILD_2:
			break;
		case MISSION_RECLAIM:
			break;
		};
		if(nb_attached>0)
			launch_script(get_script_index(SCRIPT_MoveRate2));
	}

	inline void next_mission()
	{
		if(mission==NULL) {
			set_mission(unit_manager.unit_type[type_id].DefaultMissionType,false,0,false);
			return;
			}
		switch(mission->mission)		// Commandes de fin de mission
		{
		case MISSION_REPAIR:
		case MISSION_RECLAIM:
		case MISSION_BUILD_2:
			launch_script(get_script_index(SCRIPT_stopbuilding));
			launch_script(get_script_index(SCRIPT_stop));
/*			launch_script(get_script_index("stopbuilding"));
			launch_script(get_script_index("stop"));*/
			break;
		case MISSION_ATTACK:
		case MISSION_AIM:
		case MISSION_SHOOT:
			deactivate();
			break;
		};
		if(mission->mission==MISSION_STOP && mission->next==NULL) {
			mission->data=0;
			return;
			}
		MISSION *old=mission;
		mission=mission->next;
		if(old->path)				// Détruit le chemin si nécessaire
			destroy_path(old->path);
		free(old);
		if(mission==NULL)
			set_mission(unit_manager.unit_type[type_id].DefaultMissionType);
		start_mission_script(mission->mission);
		c_time=0.0f;
	}

	inline void clear_mission()
	{
		if(mission==NULL)	return;
		MISSION *old=mission;
		mission=mission->next;
		if(old->path)				// Détruit le chemin si nécessaire
			destroy_path(old->path);
		free(old);
	}

	void add_mission(int mission_type,VECTOR *target=NULL,bool step=false,int dat=0,void *pointer=NULL,PATH_NODE *path=NULL,byte flags=0);

	void set_mission(int mission_type,VECTOR *target=NULL,bool step=false,int dat=0,bool stopit=true,void *pointer=NULL,PATH_NODE *path=NULL,byte flags=0);

	inline void compute_model_coord()
	{
		if(!compute_coord)	return;
		if(model==NULL)	return;		// S'il n'y a pas de modèle associé, on quitte la fonction
		compute_coord=false;
		MATRIX_4x4 M;
		float scale=unit_manager.unit_type[type_id].Scale;
		M=RotateZ(Angle.z*DEG2RAD)*RotateY(Angle.y*DEG2RAD)*RotateX(Angle.x*DEG2RAD)*Scale(scale);			// Matrice pour le calcul des positions des éléments du modèle de l'unité
		model->compute_coord(&data,&M);
	}

	inline void raise_signal(int signal)		// Tue les processus associés
	{
		SCRIPT_ENV_STACK *tmp;
		for(int i=0;i<nb_running;i++) {
			tmp=script_env[i].env;
			while(tmp) {
				if(tmp->signal_mask==signal) {
					tmp=script_env[i].env;
					while(tmp!=NULL) {
						script_env[i].env=tmp->next;
						delete tmp;
						tmp=script_env[i].env;
						}
					}
				if(tmp)
					tmp=tmp->next;
				}
			if(script_env[i].env==NULL)
				script_env[i].running=false;
			}
	}

	inline void init_alloc_data()
	{
		s_var=new int[15];			// Tableau de variables pour les scripts
		port=new float[21];			// Ports
		script_env=new SCRIPT_ENV[25];	// Environnements des scripts
		script_val=new short[25];	// Tableau de valeurs retournées par les scripts
		memory=new int[10];	// Pour se rappeler sur quelles armes on a déjà tiré
		script_idx=new char[NB_SCRIPT];	// Index of scripts to prevent multiple search
		attached_list=new short[20];
		link_list=new short[20];
	}

	inline void init(int unit_type=-1,int owner=-1,bool full=false)
	{
		last_time_sound = Atimer;
		cur_px=0;
		cur_py=0;
		old_sight = 0;
		severity=0;
		if(full)
			init_alloc_data();
		for(int i=0;i<NB_SCRIPT;i++)	script_idx[i]=-2;	// Not yet searched
		just_created=true;
		attached=false;
		nb_attached=0;
		mem_size=0;
		planned_weapons=0.0f;
		attacked=false;
		groupe=0;
		burst[0]=0;
		burst[1]=0;
		burst[2]=0;
		b_delay[0]=0.0f;
		b_delay[1]=0.0f;
		b_delay[2]=0.0f;
		stock[0]=0;
		stock[1]=0;
		stock[2]=0;
		h=0.0f;
		compute_coord=true;
		c_time=0.0f;
		flags=1;
		sel=false;
		script=NULL;
		model=NULL;
		owner_id=owner;
		type_id=-1;
		hp=0.0f;
		V.x=V.y=V.z=0.0f;
		Pos=V;
		data.init();
		Angle.x=Angle.y=Angle.z=0.0f;
		V_Angle=Angle;
		nb_running=0;
		int i;
		for(i=0;i<25;i++) {
			script_env[i].init();
			script_env[i].running=false;
			}
		for(i=0;i<21;i++)
			port[i]=0.0f;
		for(i=0;i<15;i++)
			s_var[i]=0;
		if(unit_type<0 || unit_type>=unit_manager.nb_unit)
			unit_type=-1;
		port[ACTIVATION]=0;
		mission=NULL;
		port[BUILD_PERCENT_LEFT]=0.0f;
		if(unit_type!=-1) {
			set_mission(MISSION_STANDBY);
			type_id=unit_type;
			model=unit_manager.unit_type[type_id].model;
			hp=unit_manager.unit_type[type_id].MaxDamage;
			script=unit_manager.unit_type[type_id].script;
			port[STANDINGMOVEORDERS]=unit_manager.unit_type[type_id].StandingMoveOrder;
			port[STANDINGFIREORDERS]=unit_manager.unit_type[type_id].StandingFireOrder;
			set_mission(unit_manager.unit_type[type_id].DefaultMissionType);
			if(script) {
				data.load(script->nb_piece);
				launch_script(get_script_index(SCRIPT_create));
//				launch_script(get_script_index("create"));
				}
			}
	}

	inline UNIT()
	{
		init();
	}

	inline void destroy(bool full=false)
	{
		for(int i=0;i<nb_running;i++)
			script_env[i].destroy();
		while(mission) clear_mission();
		init();
		if(full) {
			delete[] s_var;			// Tableau de variables pour les scripts
			delete[] port;			// Ports
			delete[] script_env;	// Environnements des scripts
			delete[] script_val;	// Tableau de valeurs retournées par les scripts
			delete[] memory;	// Pour se rappeler sur quelles armes on a déjà tiré
			delete[] script_idx;	// Index of scripts to prevent multiple search
			delete[] attached_list;
			delete[] link_list;
			}
		flags=0;
		groupe=0;
	}

	inline ~UNIT()
	{
		destroy();
	}

	void draw(CAMERA *cam,MAP *map,bool cullface=true, bool height_line=true);

	void draw_shadow(CAMERA *cam,VECTOR Dir,MAP *map);

	void draw_shadow_basic(CAMERA *cam,VECTOR Dir,MAP *map);

	inline int get_script_index(int id)
	{
		if(script_idx[id]!=-2)	return script_idx[id];
		char *script_name[]= {	"QueryPrimary","AimPrimary","FirePrimary",
								"QuerySecondary","AimSecondary","FireSecondary",
								"QueryTertiary","AimTertiary","FireTertiary",
								"TargetCleared","stopbuilding","stop",
								"startbuilding","go","killed",
								"StopMoving","Deactivate","Activate",
								"create","MotionControl","startmoving",
								"MoveRate1","MoveRate2","MoveRate3",
								"RequestState","TransportPickup","TransportDrop",
								"QueryTransport","BeginTransport","EndTransport",
								"SetSpeed","SetDirection","SetMaxReloadTime",
								"QueryBuildInfo"};
		script_idx[id]=get_script_index(script_name[id]);
		return script_idx[id];
	}

	inline int get_script_index(char *script_name)			// Cherche l'indice du script dont on fournit le nom
	{
		if(script)
			for(int i=0;i<script->nb_script;i++)
				if(strcasecmp(script->name[i],script_name)==0)
					return i;
		return -1;
	}

	inline int launch_script(int id,int nb_param=0,int *param=NULL)
	{
		if(script==NULL)	return -2;
		if(id<0 || id>=script->nb_script)
			return -2;
		if(is_running(id))			// le script tourne déjà / script already running
			return -1;
		script_env[nb_running].init();
		script_env[nb_running].env=new SCRIPT_ENV_STACK;
		script_env[nb_running].env->init();
		script_env[nb_running].env->cur=id;
		script_env[nb_running].running=true;
		if(nb_param>0 && param!=NULL)
			for(int i=0;i<nb_param;i++)
				script_env[nb_running].env->var[i]=param[i];
		return nb_running++;
	}

	inline bool is_running(int script_index)
	{
		if(script==NULL)	return false;
		if(script_index<0 || script_index>=script->nb_script)	return false;
		for(int i=0;i<nb_running;i++)
			if(script_env[i].running && script_env[i].env!=NULL) {
				SCRIPT_ENV_STACK *current=script_env[i].env;
				while(current) {
					if((current->cur&0xFF)==script_index)
						return true;
					current=current->next;
					}
				}
		return false;
	}

	inline void kill_script(int script_index)		// Fait un peu de ménage
	{
		if(script==NULL)	return;
		if(script_index<0 || script_index>=script->nb_script)	return;
		for(int i=0;i<nb_running;i++)
			if(script_env[i].running && script_env[i].env!=NULL) {
				SCRIPT_ENV_STACK *current=script_env[i].env;
				while(current) {
					if((current->cur&0xFF)==script_index) {		// Tue le script trouvé
						current=script_env[i].env;
						script_env[i].running=false;
						while(current) {
							script_env[i].env=current->next;
							delete current;
							current=script_env[i].env;
							}
						break;
						}
					current=current->next;
					}
				}
		int e=0;
		for(int i=0;i+e<nb_running;) {				// Efface les scripts qui se sont arrêtés
			if(script_env[i+e].running) {
				script_env[i]=script_env[i+e];
				i++;
				}
			else {
				script_env[i+e].destroy();
				e++;
				}
			}
		nb_running-=e;
	}

	inline void reset_script()
	{
		for(int i=0;i<nb_running;i++)
			script_env[i].destroy();
		nb_running=0;
	}

	const void run_script(const float dt,const int id,MAP *map);			// Interprète les scripts liés à l'unité

	const int move(const float dt,MAP *map,const int *path_exec,const int key_frame=0);

	void show_orders(bool only_build_commands=false);				// Dessine les ordres reçus

	inline void activate()
	{
		if(port[ACTIVATION]==0.0f) {
//			sound_manager->PlaySound( &unit_manager.unit_type[type_id].activate , &Pos );
			sound_manager->PlayTDFSound( unit_manager.unit_type[type_id].soundcategory , "activate" , &Pos );
			kill_script(get_script_index(SCRIPT_Deactivate));
			launch_script(get_script_index(SCRIPT_Activate));
			port[ACTIVATION]=1.0f;
			}
	}

	inline void deactivate()
	{
		if(port[ACTIVATION]!=0.0f) {
//			sound_manager->PlaySound( &unit_manager.unit_type[type_id].deactivate , &Pos );
			sound_manager->PlayTDFSound( unit_manager.unit_type[type_id].soundcategory, "deactivate" , &Pos );
			kill_script(get_script_index(SCRIPT_Activate));
			launch_script(get_script_index(SCRIPT_Deactivate));
			port[ACTIVATION]=0.0f;
			}
	}

	void shoot(int target,VECTOR startpos,VECTOR Dir,int w_id=0);

	bool hit(VECTOR P,VECTOR Dir,VECTOR *hit_vec, float length = 100.0f);
};

class INGAME_UNITS		// Classe pour la gestion massive des unités pendant la partie
{
public:
/*----------------------- Variables générales ----------------------------------------------*/
	uint16	nb_unit;		// Nombre d'unités
	uint16	max_unit;		// Nombre maximum d'unités stockables dans le tableau
	UNIT	*unit;			// Tableau contenant les références aux unités
	uint16	index_list_size;
	uint16	*idx_list;
	uint16	*free_idx;
	uint16	free_index_size[10];

/*----------------------- Variables réservées au joueur courant ----------------------------*/

	float	nb_attacked;
	float	nb_built;

	uint8	page;

/*----------------------- Variables reserved to precalculations ----------------------------*/

	float	exp_dt_1;
	float	exp_dt_2;
	float	exp_dt_4;
	float	g_dt;
	int		sound_min_ticks;

	inline void init()
	{
		sound_min_ticks = MSEC_TO_TIMER( 500 );
		index_list_size=0;
		for(int i=0;i<10;i++)	free_index_size[i]=0;
		idx_list=free_idx=NULL;
		page=0;
		nb_unit=0;
		unit=NULL;
		max_unit=0;
		nb_attacked=0.0f;
		nb_built=0.0f;
		exp_dt_1=0.0f;
		exp_dt_2=0.0f;
		exp_dt_4=0.0f;
		g_dt=0.0f;
	}

	inline INGAME_UNITS()
	{
		init();
	}

	inline void destroy()
	{
		if(idx_list)			delete[] idx_list;
		if(free_idx)			delete[] free_idx;
		if(max_unit>0 && unit)			// Détruit une à une les unités
			for(int i=0;i<max_unit;i++)
				unit[i].destroy(true);
		if(unit)
			free(unit);
		init();
	}

	inline ~INGAME_UNITS()
	{
		destroy();
	}

	void kill(int index,MAP *map,int prev);			// Détruit une unité

	void draw(CAMERA *cam,MAP *map,bool underwater=false,bool limit=false,bool cullface=true,bool height_line=true);					// Dessine les unités visibles

	void draw_shadow(CAMERA *cam,VECTOR Dir,MAP *map,float alpha=0.5f);					// Dessine les ombres des unités visibles

	void draw_mini(float map_w,float map_h,int mini_w,int mini_h,SECTOR **map_data);				// Repère les unités sur la mini-carte

	void move(float dt,MAP *map=NULL,int key_frame=0,bool wind_change=false);

	int create(int type_id,int owner);

	bool select(CAMERA *cam,int sel_x[],int sel_y[]);

	int pick(CAMERA *cam,int sensibility=1);

	inline void give_order_move(int player_id,VECTOR target,bool set=true,byte flags=0)
	{
		for(uint16 e=0;e<index_list_size;e++) {
			uint16 i = idx_list[e];
			if(unit[i].flags!=0 && unit[i].owner_id==player_id && unit[i].sel && unit[i].port[BUILD_PERCENT_LEFT]==0.0f && unit_manager.unit_type[unit[i].type_id].canmove) {
				if(set)
					unit[i].set_mission(MISSION_MOVE_CALC_PATH,&target,false,0,true,NULL,NULL,flags);
				else
					unit[i].add_mission(MISSION_MOVE_CALC_PATH,&target,false,0,NULL,NULL,flags);
				if(player_id==players.local_human_id && Atimer-unit[i].last_time_sound > sound_min_ticks ) {
					unit[i].last_time_sound = Atimer;
					sound_manager->PlayTDFSound( unit_manager.unit_type[unit[i].type_id].soundcategory, "ok1" , &unit[i].Pos );
					}
				}
			}
	}

	inline void give_order_patrol(int player_id,VECTOR target,bool set=true)
	{
		for(uint16 e=0;e<index_list_size;e++) {
			uint16 i = idx_list[e];
			if(unit[i].flags!=0 && unit[i].owner_id==player_id && unit[i].sel && unit[i].port[BUILD_PERCENT_LEFT]==0.0f && unit_manager.unit_type[unit[i].type_id].canpatrol) {
				if(set)
					unit[i].set_mission(MISSION_PATROL,&target,false,0,true,NULL,NULL);
				else
					unit[i].add_mission(MISSION_PATROL,&target,false,0,NULL,NULL);
				if(player_id==players.local_human_id && Atimer-unit[i].last_time_sound > sound_min_ticks ) {
					unit[i].last_time_sound = Atimer;
					sound_manager->PlayTDFSound( unit_manager.unit_type[unit[i].type_id].soundcategory, "ok1" , &unit[i].Pos );
					}
				}
			}
	}

	inline void give_order_guard(int player_id,int target,bool set=true)
	{
		for(uint16 e=0;e<index_list_size;e++) {
			uint16 i = idx_list[e];
			if(unit[i].flags!=0 && unit[i].owner_id==player_id && unit[i].sel && unit[i].port[BUILD_PERCENT_LEFT]==0.0f && unit_manager.unit_type[unit[i].type_id].canguard) {
				if(set)
					unit[i].set_mission(MISSION_GUARD,&unit[target].Pos,false,0,true,&(unit[target]),NULL);
				else
					unit[i].add_mission(MISSION_GUARD,&unit[target].Pos,false,0,&(unit[target]),NULL);
				if(player_id==players.local_human_id && Atimer-unit[i].last_time_sound > sound_min_ticks ) {
					unit[i].last_time_sound = Atimer;
					sound_manager->PlayTDFSound( unit_manager.unit_type[unit[i].type_id].soundcategory, "ok1" , &unit[i].Pos );
					}
				}
			}
	}

	inline void give_order_load(int player_id,int target,bool set=true)
	{
		if(unit[target].flags==0 || !unit_manager.unit_type[unit[target].type_id].canmove)	return;
		switch(unit_manager.unit_type[unit[target].type_id].TEDclass)
		{
		case CLASS_UNDEF:
		case CLASS_WATER:
		case CLASS_SHIP:
		case CLASS_PLANT:
		case CLASS_SPECIAL:
		case CLASS_FORT:
			return;
			break;
		};
		for(uint16 e=0;e<index_list_size;e++) {
			uint16 i = idx_list[e];
			if(unit[i].flags!=0 && unit[i].owner_id==player_id && unit[i].sel && unit[i].port[BUILD_PERCENT_LEFT]==0.0f && unit_manager.unit_type[unit[i].type_id].canload) {
				if(set)
					unit[i].set_mission(MISSION_LOAD,&unit[target].Pos,false,0,true,&(unit[target]),NULL);
				else
					unit[i].add_mission(MISSION_LOAD,&unit[target].Pos,false,0,&(unit[target]),NULL);
				if(player_id==players.local_human_id && Atimer-unit[i].last_time_sound > sound_min_ticks ) {
					unit[i].last_time_sound = Atimer;
					sound_manager->PlayTDFSound( unit_manager.unit_type[unit[i].type_id].soundcategory, "ok1" , &unit[i].Pos );
					}
				}
			}
	}

	inline void give_order_build(int player_id,int unit_type_id,VECTOR target,bool set=true)
	{
		for(uint16 e=0;e<index_list_size;e++) {
			uint16 i = idx_list[e];
			if(unit[i].flags!=0 && unit[i].owner_id==player_id && unit[i].sel && unit[i].port[BUILD_PERCENT_LEFT]==0.0f && unit_manager.unit_type[unit[i].type_id].Builder) {
				if(player_id==players.local_human_id && Atimer-unit[i].last_time_sound > sound_min_ticks ) {
					unit[i].last_time_sound = Atimer;
					sound_manager->PlayTDFSound( unit_manager.unit_type[unit[i].type_id].soundcategory, "build" , &unit[i].Pos );
					}
				if(set)
					unit[i].set_mission(MISSION_BUILD,&target,false,unit_type_id);
				else
					unit[i].add_mission(MISSION_BUILD,&target,false,unit_type_id);
				}
			}
	}

#define SIGNAL_ORDER_NONE	0x0
#define SIGNAL_ORDER_MOVE	0x1
#define SIGNAL_ORDER_PATROL	0x2
#define SIGNAL_ORDER_GUARD	0x3
#define SIGNAL_ORDER_ATTACK	0x4
#define SIGNAL_ORDER_RECLAM	0x5
#define SIGNAL_ORDER_STOP	0x6
#define SIGNAL_ORDER_ONOFF	0x7
#define SIGNAL_ORDER_LOAD	0x8
#define SIGNAL_ORDER_UNLOAD	0x9

	int menu_order(int omb,int player_id);							// Menu permettant de donner des ordres

	void complete_menu(int index);
};

extern INGAME_UNITS units;

const bool can_be_built(const VECTOR Pos,MAP *map,const int unit_type_id);

#endif
